#!/usr/bin/env python3
"""
NetorcaBaseModule is a base class for all Netorca Ansible modules. It provides common functionality
and helper methods for modules to interact with the Netorca API.

This class handles tasks such as authentication, connection, and error handling.

Copyright: (c) 2022, NetAutomate <info@netautomate.org>
"""
from __future__ import absolute_import, division, print_function

import logging
from typing import Any, Dict, List, Union

from netorca_sdk.auth import NetorcaAuth
from netorca_sdk.netorca import Netorca
from validators import url as url_valid

from ..module_utils.netorca_constants import (
    FIELDS_API_KEY,
    FIELDS_CONTEXT,
    FIELDS_FILTERS,
    FIELDS_PASS,
    FIELDS_URL,
    FIELDS_USER,
    VALID_FILTERS,
)

__metaclass__ = type

logging.basicConfig(filename=__name__ + ".log", level=logging.DEBUG)


class NetorcaModuleError(Exception):
    pass


class NetorcaModuleValidationError(NetorcaModuleError):
    pass


class NetorcaBaseModule:
    RESULT_FIELD_SI = None
    RESULT_FIELD_MESSAGE = "message"
    RESULT_FIELD_CHANGED = "changed"

    def __init__(self, module: Any):
        self.module = module
        self.output_dict = {
            self.RESULT_FIELD_SI: [],
            self.RESULT_FIELD_CHANGED: False,
            self.RESULT_FIELD_MESSAGE: "Starting Module",
        }

    def fail_module(self, msg: str) -> None:
        """Fail the module with the given error message."""
        result = {self.RESULT_FIELD_SI: []}
        logging.error(msg)
        self.module.fail_json(msg=msg, result=result)

    def validate_params(self) -> None:
        """Validate the input parameters."""
        params = self.module.params
        keys = params.keys()

        # Check that either api_key or username + password supplied
        if FIELDS_API_KEY not in keys:
            if not (FIELDS_USER in keys and FIELDS_PASS in keys):
                raise NetorcaModuleValidationError("If no api_key specified, username and password required")

        # Check that URL is a valid URL
        if not url_valid(params[FIELDS_URL]):
            raise NetorcaModuleValidationError(f"{params[FIELDS_URL]} is not a valid url")

    def connect(self) -> None:
        """Connect to the Netorca service."""
        try:
            self.validate_params()
        except NetorcaModuleValidationError as e:
            self.fail_module(str(e))

        # Authenticate
        if FIELDS_API_KEY not in self.module.params or not self.module.params[FIELDS_API_KEY]:
            logging.debug("No API key provided, logging in")
            netorca_auth = NetorcaAuth(
                username=self.module.params[FIELDS_USER],
                password=self.module.params[FIELDS_PASS],
                fqdn=self.module.params[FIELDS_URL],
            )
        else:
            logging.debug("API key provided, skipping logging in")
            netorca_auth = NetorcaAuth(api_key=self.module.params[FIELDS_API_KEY], fqdn=self.module.params[FIELDS_URL])

        self.netorca = Netorca(auth=netorca_auth)

    def make_filter(self) -> Dict[str, str]:
        """Create a filter based on the input parameters."""
        filters = self.module.params.get(FIELDS_FILTERS, None)

        # TODO: Implement filter fields validation
        # for filter_key in VALID_FILTERS:
        #     if filter_key in self.module.params.keys() and self.module.params[filter_key]:
        #         filters[filter_key] = self.module.params[filter_key]

        return filters

    def get_context(self) -> Union[Dict[str, Any], None]:
        """Retrieve the context from the input parameters."""
        context = None
        if FIELDS_CONTEXT in self.module.params.keys() and self.module.params[FIELDS_CONTEXT]:
            context = self.module.params.pop(FIELDS_CONTEXT)
        return context

    def run_module(self, func: str, method: str, id: str = None, data: dict = None) -> None:
        """Execute the given function and handle any errors."""
        try:
            self.connect()
            filters = self.make_filter()
            context = self.get_context()
            result_list = self.netorca.caller(func, method, filters=filters, context=context, id=id, data=data)
            if isinstance(result_list, list):
                self.make_response(result_list, as_list=True)
            else:
                self.make_response(result_list)
        except Exception as e:
            self.fail_module(f"An error occurred: {str(e)}")

    def make_response(self, result: Union[List, dict], as_list: bool = False) -> None:
        """Create the response to be returned by the module."""
        self.output_dict[self.RESULT_FIELD_SI] = result["results"] if as_list else result
        self.output_dict[self.RESULT_FIELD_MESSAGE] = f"Returned {len(result)} items"
        self.module.exit_json(**self.output_dict)



#!/usr/bin/env python3

from ansible.module_utils.basic import AnsibleModule

from ..module_utils.netorca_common import NetorcaBaseModule, NetorcaModuleValidationError
from ..module_utils.netorca_constants import (
    FIELDS_API_KEY,
    FIELDS_CONTEXT,
    FIELDS_OBJECT_ID,
    FIELDS_PASS,
    FIELDS_SERVICE,
    FIELDS_STATE,
    FIELDS_URL,
    FIELDS_USER,
    NETORCA_VALID_UPDATE_STATES,
)

DOCUMENTATION = r"""
---
module: update_change_instances

short_description: Update Change Instances in NetOrca

version_added: "0.1.0"

description: This module updates the change instances for a team in NetOrca.

options:
    url:
        description: Base URL for NetOrca.
        required: true
        type: str
    api_key:
        description: API Key generated for the team. If this is not specified, username and password need to be provided.
        required: false
        type: str
    username:
        description: Username for the account in the team. Use when no API KEY is available.
        required: false
        type: str
    password:
        description: Password for the account in the team. Use when no API KEY is available.
        required: false
        type: str
    uuid:
        description: The ID of the change instance to update.
        required: false
        type: str
    state:
        description: One of 'PENDING', 'ACCEPTED', 'COMPLETED'.
        required: false
        type: str

extends_documentation_fragment:
    - netautomate.netorca.update_change_instances

author:
    - Scott Rowlandson
"""

EXAMPLES = r"""
# Supply API Key and update change instance
- name: Update change instance
  update_change_instances:
    url: https://app.netorca.io
    api_key: <api key here>
    uuid: <change instance ID here>
    state: ACCEPTED

# Supply credentials and update change instance
- name: Update change instance
  netorca_update_changes:
    url: https://app.netorca.io
    username: team_member_a
    password: super_secure_password_here
    uuid: <change instance ID here>
    state: COMPLETED
"""

RETURN = r"""
# These are examples of possible return values, and in general, should use other names for return values.
updated_change_instances:
    description: The change instance returned by NetOrca.
    type: list
    returned: always
message:
    description: A general message, useful when errors are encountered.
    type: str
    returned: always
    sample: 'Updated change instance successfully'
"""


class UpdateChangeInstances(NetorcaBaseModule):
    RESULT_FIELD_SI = "updated_change_instances"

    def validate_params(self) -> None:
        super().validate_params()

        # Check that State is one of valid states
        if FIELDS_STATE in self.module.params:
            if not self.module.params[FIELDS_STATE] in NETORCA_VALID_UPDATE_STATES:
                raise NetorcaModuleValidationError("'state' is not one of {NETORCA_VALID_UPDATE_STATES}")


FIELDS = {
    FIELDS_URL: dict(type="str", required=True),
    FIELDS_API_KEY: dict(type="str", required=False, no_log=True),
    FIELDS_USER: dict(type="str", required=False),
    FIELDS_PASS: dict(type="str", required=False, no_log=True),
    FIELDS_SERVICE: dict(type="str", required=False),
    FIELDS_CONTEXT: dict(type="str", required=False),
    FIELDS_OBJECT_ID: dict(type="str", required=False),
    FIELDS_STATE: dict(type="str", required=False),
}


def main() -> None:
    module = AnsibleModule(argument_spec=FIELDS, supports_check_mode=True)

    try:
        UpdateChangeInstances(module).run_module(
            "change_instances",
            "update",
            id=module.params[FIELDS_OBJECT_ID],
            data={FIELDS_STATE: module.params[FIELDS_STATE]},
        )
    except Exception as e:
        module.fail_json(msg=f"An error occurred: {str(e)}")


if __name__ == "__main__":
    main()



import unittest
from unittest.mock import patch

from netorca.plugins.repository import NetorcaBaseModule, NetorcaModuleValidationError


class MockNetorcaAuth:
    def __init__(self, username=None, password=None, fqdn=None, api_key=None):
        pass


class MockNetorca:
    def __init__(self, auth=None):
        pass


class MockAnsibleModule:
    def __init__(self, argument_spec=None):
        self.params = {
            "api_key": "test_key",
            "url": "https://example.com",
        }

    def fail_json(self, *args, **kwargs):
        raise Exception("Module failed")


class TestNetorcaBaseModule(unittest.TestCase):
    def setUp(self):
        self.mock_module = MockAnsibleModule()
        self.base_module = NetorcaBaseModule(self.mock_module)

    @patch("netorca_sdk.netorca.Netorca", new=MockNetorca)
    @patch("netorca_sdk.auth.NetorcaAuth", new=MockNetorcaAuth)
    def test_connect_with_api_key(self):
        try:
            self.base_module.connect()
        except Exception as e:
            self.fail(f"connect() raised an exception: {e}")

    def test_make_filter(self):
        self.mock_module.params["service_name"] = "test_value"
        expected_filter = {"service_name": "test_value"}
        self.assertEqual(self.base_module.make_filter(), expected_filter)

    def test_get_context(self):
        self.mock_module.params["context"] = {"key": "value"}
        expected_context = {"key": "value"}
        self.assertEqual(self.base_module.get_context(), expected_context)

    def test_fail_module(self):
        with self.assertRaises(Exception) as context:
            self.base_module.fail_module("Test failure")
        self.assertTrue("Module failed" in str(context.exception))

    def test_validate_params_without_api_key_and_username_password(self):
        self.mock_module.params = {
            "url": "https://example.com",
        }
        with self.assertRaises(NetorcaModuleValidationError) as context:
            self.base_module.validate_params()
        self.assertTrue("If no api_key specified, username and password required" in str(context.exception))

    def test_validate_params_with_invalid_url(self):
        self.mock_module.params = {
            "api_key": "test_key",
            "url": "invalid_url",
        }
        with self.assertRaises(NetorcaModuleValidationError) as context:
            self.base_module.validate_params()
        self.assertTrue("is not a valid url" in str(context.exception))


if __name__ == "__main__":
    unittest.main()



"""'
Contains the Constants used by myansible modules
"""
NETORCA_STATES_ERRORED = "ERROR"
NETORCA_STATES_PENDING = "PENDING"
NETORCA_STATES_APPROVED = "APPROVED"
NETORCA_STATES_COMPLETED = "COMPLETED"
NETORCA_VALID_STATES = [
    NETORCA_STATES_ERRORED,
    NETORCA_STATES_PENDING,
    NETORCA_STATES_APPROVED,
    NETORCA_STATES_COMPLETED,
]
NETORCA_VALID_UPDATE_STATES = [NETORCA_STATES_ERRORED, NETORCA_STATES_APPROVED, NETORCA_STATES_COMPLETED]
NETORCA_VALID_CONTEXTS = ["owner", "consumer"]

FIELDS_URL = "url"
FIELDS_USER = "username"
FIELDS_API_KEY = "api_key"
FIELDS_PASS = "password"
FIELDS_STATE = "state"
FIELDS_SERVICE = "service_name"
FIELDS_DEPLOYED_ITEM = "deployed_item"
FIELDS_CONTEXT = "context"
FIELDS_OBJECT_ID = "uuid"
FIELDS_FILTERS = "filters"

VALID_FILTERS = [
    FIELDS_SERVICE,
]



from setuptools import setup, find_packages

setup(
    name='netautomate.netorca',
    version='1.0.9',
    author='Scott Rowlandson',
    author_email='scott@netautomate.org',
    description='Ansible modules to interact with NetOrca',
    packages=find_packages(),
    install_requires=[
        'ansible-base',
    ],
)


#!/usr/bin/env python3
from ansible.module_utils.basic import AnsibleModule

from ..module_utils.netorca_common import NetorcaBaseModule
from ..module_utils.netorca_constants import (
    FIELDS_API_KEY,
    FIELDS_CONTEXT,
    FIELDS_FILTERS,
    FIELDS_OBJECT_ID,
    FIELDS_PASS,
    FIELDS_SERVICE,
    FIELDS_URL,
    FIELDS_USER,
)

DOCUMENTATION = r"""
---
module: get_services

short_description: Get Services form NetOrca

version_added: "0.1.0"

description: This modules gets the services for a team in Netorca

options:
    url:
        description : Base URL for NetOrca.
        required: true
        type: str
    api_key:
        description : API Key generataed for tema. If this is not specified, \
            username and password need to be provided.
        required: false
        type: str
    username:
        description : Username for account in the team. Use when no API KEY is \
            available.
        required: false
        type: str
    password:
        description : Password for account in the team. Use when no API KEY is \
            available.
        required: false
        type: str
    service_name:
        description : The name of the service
        required: true
        type: str

extends_documentation_fragment:
    - netautomate.netorca.get_services

author:
    - Scott Rowlandson
"""

EXAMPLES = r"""
# Supply API Key and get all changes
- name: Get all service items
  get_service:
    url: https://app.netorca.io
    api_key: <api key here>
    service_name: LoadBalancer
"""

RETURN = r"""
# These are examples of possible return values, and in general should use other names for return values.
services:
    description: An array of all the available services
    type: array
    returned: always
message:
    description: A general message, useful when errors are encountered
    type: str
    returned: always
    sample: 'Returned 10 items'
"""


class GetServices(NetorcaBaseModule):
    RESULT_FIELD_SI = "services"


FIELDS = {
    FIELDS_URL: dict(type="str", required=True),
    FIELDS_API_KEY: dict(type="str", required=False, no_log=True),
    FIELDS_USER: dict(type="str", required=False),
    FIELDS_PASS: dict(type="str", required=False, no_log=True),
    FIELDS_SERVICE: dict(type="str", required=False),
    FIELDS_CONTEXT: dict(type="str", required=False),
    FIELDS_FILTERS: dict(type="dict", required=False),
    FIELDS_OBJECT_ID: dict(type="str", required=False),
}


def main() -> None:
    module = AnsibleModule(argument_spec=FIELDS, supports_check_mode=True)

    try:
        GetServices(module).run_module("services", "get", id=module.params.get(FIELDS_OBJECT_ID, None))
    except Exception as e:
        module.fail_json(msg=f"An error occurred: {str(e)}")


if __name__ == "__main__":
    main()
